/**
 * @file Ce fichier définit le routage d'URL au sein de l'interface du _backend_.
 * Il définit la page de connexion `/`, le panneau administrateur `/admin` et l'API REST \(`/db/:table?`)
 * permettant de consulter la base de donnée interne à Sigma, via des requêtes construites avec Knex.
 * @author manifold
 * 
 * Les res.redirect() sont censes supporter les paths relatifs (et donc pas besoin de repreciser /* a chaque fois)
 * mais ca marche visiblement pas... Donc j'ai mis les paths absolus dans les res.redirect().
 */
import express from 'express';
import knex from '../../db/knex_router';
import passport from 'passport';
import { ensureLoggedIn } from 'connect-ensure-login';

const router = express.Router();

let port = process.env.PORT || 3000;


/**
 * @description Le login se fait en POST. Faire un GET à la racine / renvoie sur
 * /login ou sur /admin selon que l'utilisateur est connecté ou non.
 */

router.get('/', function (req, res) {
    console.log("GET handler for / route");
    console.log('Connecting to ' + req.url);
    console.log('Trying to go to admin page...');
    res.redirect('/admin'); 
});

router.get('/login', function (req, res) {
    console.log('Connecting to ' + req.url);
    res.render('login', { title: 'Login', port: port, 
        errorMessage: req.flash('error') }); //lets pug render src/views/login.pug with specified attributes
});

router.get('/admin',
    ensureLoggedIn('/login'),
    function (req, res) {
        console.log('Connecting to ' + req.url);
        let userName;
        // Une erreur a ce stade peut etre triggered si req.user n'existe pas
        // mais pour autant on est assures que la personne est bien authentifiee
        // donc on laisse passer sans déclencher d'erreur 500
        try {
            let user = req.user;
            //let user = req.user;
            console.log('Welcome,',user.uid);
            userName = user.uid;
        } catch (err) {
            console.log("Warning: in admin_router router.get('/admin')");
            console.log(err.message);
            userName = "No one";
        }
        res.render('home', { title: 'Home', port: port, userName: userName });
    }
);

router.post('/logout', function (req, res) {
    req.logout();
    res.redirect('/');
});

// je pense qu'on ferait mieux d'utiliser ca
// https://expressjs.com/en/4x/api.html#router.route
router.get('/db?', function (req, res) {
    let table_name = req.query.table;
    let columns = req.query.columns;

    res.redirect(`/db/${table_name}?columns=${columns}`);
});

/**
 * @function Knex API: Get table
 * @summary Effectue une requête pour une table dans la BDD
 * @argument {string} table_name - La table voulue par l'utilisateur.
 */
router.get('/db/:table_name?', function (req, res) {

    // get columns from query
    let columns;
    if (req.query.columns) {
        columns = req.query.columns.split(',');
    } else {
        columns = null;
    }
    console.log(columns);

    knex.select(columns).from(req.params.table_name).then(function (table) {
        res.setHeader("Content-Type", "application/json");
        res.write(JSON.stringify(table, null, 2));
        res.end();
    }, function () {
        res.status(400);
        res.render('error', {
            status: res.statusCode,
            error_message: "Bad request: can't find table " + req.params.table_name
        });
        res.end();
    }
    );
});

router.post('/login', (req, res, next) => {
    passport.authenticate('ldapauth', (err, user, info) => {
        if (err) return next(err); // handle error
        else if (!user) {
            return res.json(403, {message: "Not authenticated."});
        }

        console.log(req.headers); // return 'Origin' header, useful later
        
        req.login(user, (err) => {
            if (err) {
                console.log(err);
                return err;
            }
            // if request asks for a json, reply with a token
            // else redirect to admin panel
            (req.header('accept') == 'application/json')
                ? res.json({ message: 'Authentication succeeded.' })
                : res.redirect('/admin');
            
        });
    })(req,res,next);
});

/**
 * @function Error 404 catcher
 * @summary Catche les requêtes en dehors des URL acceptées
 */
router.use((req, res, next) => {
    let err = new Error('Not found');
    err.status = 404;
    next(err);
});

/**
 * @function Error 404 handler
 * @summary Gère les erreurs 404
 */
router.use((err, req, res, next) => {
    console.log("Entering error handler");
    res.locals.message = err.message;
    console.log(err.message);

    res.status(err.status || 500);
    let error_message = res.statusCode == 404 ? 'Not found.' : 'Internal server error.';
    res.render('error', {
        status: res.statusCode,
        error_message: error_message
    });
});

export default router;
